package application;

import debug.*;
import timer.*;
import link.*;
import network.*;
import reliability.*;

import java.io.*;
import java.util.*;

/**
  A class to act as an application to drive the receiving end of a 
  bytestream conversation.  Matches up with a 
  SimpleBytestreamSender on the other end, and works through
  a ReliableReceiver instance to acheive communication.
  */

public class SimpleBytestreamReceiver implements Runnable
{
  private ReliableReceiver receiver;
  private Address addr;
  private int numBytes; 

  /**
    Creates a new recevier client and starts the run thread for 
    it.  Clients should not be made to share receiver instances,
    because it probably won't work.
    
    @param receiveImplementation the ReliableReceiver instance
           (which needs to be connected to a network and have
           an address set) through which to receive

    @param address the address of this host.  Used to name
           the receiving thread and for messages

    @param numBytes number of bytes to receive
    */
  public SimpleBytestreamReceiver (ReliableReceiver receiveImplementation, 
				   Address address, 
				   int numBytes)
  {
    receiver = receiveImplementation;
    addr = address;
    this.numBytes = numBytes; 
    
    Thread sendThread = new Thread (this);
    Timer.global.RegisterThread (sendThread);
    sendThread.start ();
  }
  
  private static int CHUNKSIZE = 20;
  
  /*
    The receiving thread.  Clients should not call this: the
    thread is started automatically by the constructor
    */
  public void run ()
  {
    Thread.currentThread ().setName ("Recevier application at " + addr);
    
    synchronized (Timer.global)
      {
	long oldTime = Timer.global.CurrentSimTime ();
	long newTime = oldTime;
	int linesReceived = 0;
	
	while (linesReceived*CHUNKSIZE < numBytes) { 
	  byte[] data = receiver.Receive (CHUNKSIZE);
	  if (data.length < CHUNKSIZE)
	    return;
	  
	  newTime = Timer.global.CurrentSimTime ();
	  
	  ++linesReceived;
	  
	  String s = new String (data, 0);
	  /*
	    System.out.println ("" + Timer.global.CurrentSimTime () + ": " + 
	    addr + ": '" + s + "' (" + 
	    (8 * CHUNKSIZE * linesReceived)+" bits in "+ 
	    (newTime - oldTime) + " us = "
	    + (float)((8.0 * CHUNKSIZE * linesReceived)
	    / (newTime - oldTime)) + " Mb/sec)");
	    */
	  System.out.println ( "(" + 
			       (8 * CHUNKSIZE * linesReceived)+" bits in "+ 
			       (newTime - oldTime) + " us = "
			       + (float)((8.0 * CHUNKSIZE * linesReceived)
					 / (newTime - oldTime)) + " Mb/sec)");
	}
	
	System.out.println ( "(" + 
			    (8 * CHUNKSIZE * linesReceived)+" bits in "+ 
			    (newTime - oldTime) + " us = "
			    + (float)((8.0 * CHUNKSIZE * linesReceived)
				      / (newTime - oldTime)) + " Mb/sec)");

	Timer.global.UnregisterThread (Thread.currentThread ());
      }
  }
  
}






